// ConsoleVisualiser.h
// A console visualiser for the FLIP language.
// A. Savidis, October, 2005.
// Project: FLIP Suite (Execution visualizer, Savidis, 2005).
//

#ifndef	CONSOLEVISUALISER_H
#define	CONSOLEVISUALISER_H

#if	defined(HAS_VISUALIZER) && defined(CONSOLE_VISUALIZER)

#ifdef	GTK_VISUALIZER
#error "Should define either GTK_VISUALIZER or CONSOLE_VISUALIZER!"
#endif

#include "VisualiserSuper.h"
#include <stdio.h>
#include <assert.h>

class ConsoleVisualiser : public VisualiserSuper {

	////////////////////////////////////////////////////////////

	private:
	static const char*	Input (const char* buff, unsigned len, FILE* fp);
	static int			Output (const char* format, ...);

	static void			CleanUp (void);

	static void			OnProgramStart (void);
	static void			OnProgramEnd (void);
	static void			OnProgramError (const char* s);
	static void			OnProgramStop (unsigned line);

	static bool			fileOutput;
	static FILE*		file;

	//**************************

#if	defined(VISUALISER_FILEOUTPUT) && !defined(VISUALISER_FILE)
#define	VISUALISER_FILE	"servanttrace.txt"
#endif

	static void PreInitialiser (void) {
#ifdef	VISUALISER_FILEOUTPUT
		fileOutput = true;
		file = fopen(VISUALISER_FILE, "wt");
		if (!file)
			fileOutput = false;
#endif
	}

	static void PostCleaner (void) {
#ifdef	VISUALISER_FILEOUTPUT
		if (file) {
			assert(fileOutput);
			fclose(file);
		}
#endif
	}

	////////////////////////////////////////////////////////////
	
	public:
	void Message (const char* str) const;

	void CellRefresh (
			const char*		name,	
			const char*		content,
			const char*		type, 
			const char*		storage, 
			const char*		operation, 
			unsigned		page, 
			unsigned		row
		);

	void CellErase (
			unsigned	page, 
			unsigned	row
		);

	void StepWait (void) const { /* Can't step in console mode. */ }

	ConsoleVisualiser (void);
	~ConsoleVisualiser();
};


///////////////////////////////////////////////////////

#endif	// Header file end point.

#endif	// HAS_VISUALIZER && CONSOLE_VISUALIZER
