// GraphicalVisualiser.h
// A Graphical visualiser for the FLIP language.
// A. Savidis, October, 2005.
// Deep re-factoring of first version from Y. Lilis.
// Project: FLIP Suite (Execution visualizer, Savidis, 2005).
//

#ifndef	GRAPHICALVISUALISER_H
#define	GRAPHICALVISUALISER_H

#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#ifdef	CONSOLE_VISUALISER
#error "Should define either GRAPHICAL_VISUALIZER or CONSOLE_VISUALIZER!"
#endif

#include "VisualiserSuper.h"
#include <stdio.h>
#include <assert.h>

////////////////////////////////////////////////////////////

class GraphicalVisualiser : public VisualiserSuper {

	////////////////////////////////////////////////////////////

	private:
	static const char*	Input (const char* buff, unsigned len, FILE* fp);
	static int			Output (const char* format, ...);

	static void			CleanUp (void);

	static void			OnProgramStart (void);
	static void			OnProgramEnd (void);
	static void			OnProgramError (const char* s);
	static void			OnProgramStop (unsigned line);

	//*****************************

	static void PreInitialiser (void);
	static void PostCleaner (void);
	
	unsigned eventMutex;

	////////////////////////////////////////////////////////////
	
	public:
	
	// Thread enabled versions for the graphical visualiser that works in parallel.
	//
	virtual bool IsNotifyingEvent (const std::string& name);

	virtual void SetNotifyingEvent (const std::string& name, bool enabled);

	virtual bool IsSteppingEvent (const std::string& name);

	virtual void SetSteppingEvent (const std::string& name, bool enabled);

	//*****************************

	void Message (const char* str) const;

	void CellRefresh (
			const char*		name,	
			const char*		content,
			const char*		type, 
			const char*		storage, 
			const char*		operation, 
			unsigned		page, 
			unsigned		row
		);

	void CellErase (
			unsigned	page, 
			unsigned	row
		);

	void StepWait (void) const;

	GraphicalVisualiser (void);
	~GraphicalVisualiser();
};


///////////////////////////////////////////////////////

#endif	// Header file end point.

#endif	// HAS_VISUALIZER && CONSOLE_VISUALIZER
