// NotebookMirror.h
// Notebook content mirroring for access by the the Graphical UI
// an the Graphical visualiser.
// Graphical visualiser for the FLIP language.
// A. Savidis, October, 2005.
//

#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#ifndef	NOTEBOOKMIRROR_H
#define	NOTEBOOKMIRROR_H

#include <map>
#include <string>

class NotebookMirror {

	friend class GraphicalVisualiser;

	private:
	////////////////////////////////////////////////////////

	static NotebookMirror* singletonPtr;
	static void	SingletonCreate (void);
	static void	SingletonDestroy (void);

	//**********************

	struct Cell {

		std::string name, value, type, storage;

		void operator=(const Cell& cell) 
			{ new (this) Cell(cell); }

		Cell (
			const std::string& _name,
			const std::string& _value,
			const std::string& _type,
			const std::string& _storage
		) : name (_name), value(_value), type(_type), storage(_storage){}

		Cell (const Cell& c) : name (c.name), value(c.value), type(c.type), storage(c.storage){}

		Cell (void){ Cell::Cell("", "", "", ""); }

		~Cell(){}
	};

	//**********************

	typedef std::map<unsigned, Cell> CellMap;
	CellMap		cells;
	unsigned	cellsMutex;

	Cell* GetCell (unsigned page, unsigned row);

	Cell* NewCell (unsigned page, unsigned row);
	
	////////////////////////////////////////////////////////

	public:
	static NotebookMirror*	GetSingleton (void);

	void		GrantCellAccess (void);
	void		ReleaseCellAccess (void);

	bool		IsCellOccupied (unsigned page, unsigned row);

	void		GetCellContent (
					unsigned		page,
					unsigned		row,
					std::string&	name,
					std::string&	value,
					std::string&	type,
					std::string&	storage
				);

	void		EraseCell (unsigned page, unsigned row);

	void		SetCellContent (
					unsigned			page,
					unsigned			row,
					const std::string&	name,
					const std::string&	value,
					const std::string&	type,
					const std::string&	storage
				);

	NotebookMirror (void);
	~NotebookMirror();
};

#endif	// Header file end point.

#endif	// HAS_VISUALIZER && GRAPHICAL_VISUALIZER

