// ExecutionEvents.cpp
// Variables for ExecutionEvents class.
// A. Savidis, September, 2005.
// Project: FLIP Execution Visualizer.
//

#ifdef	HAS_VISUALIZER

#include "ExecutionEvents.h"

void (*ExecutionEvents::onBlockEnter)(unsigned line);
void (*ExecutionEvents::onBlockExit)(unsigned line);
void (*ExecutionEvents::onIfEnter)(unsigned line);
void (*ExecutionEvents::onIfExit)(unsigned line);
void (*ExecutionEvents::onForEnter)(unsigned line);
void (*ExecutionEvents::onForExit)(unsigned line);
void (*ExecutionEvents::onWhileEnter)(unsigned line);
void (*ExecutionEvents::onWhileExit)(unsigned line);
void (*ExecutionEvents::onBreak)(unsigned line);
void (*ExecutionEvents::onContinue)(unsigned line);
void (*ExecutionEvents::onCaseBegin)(unsigned line);
void (*ExecutionEvents::onCaseEnd)(unsigned line);
void (*ExecutionEvents::onConditionValue)(const char* stmt, bool val);

void (*ExecutionEvents::onArithmeticOperatorUsage)(
	const char*		op, 
	unsigned		page1, // arg1
	unsigned		row1, 
	unsigned		page2, // arg2
	unsigned		row2
);

void (*ExecutionEvents::onRelationalOperatorUsage)(
	const char*		op, 
	unsigned		page1, // arg1
	unsigned		row1, 
	unsigned		page2, // arg2
	unsigned		row2
);

void (*ExecutionEvents::onUnaryOperatorUsage)(
	const char*		op, 
	unsigned		page, 
	unsigned		row
);

//*************************

void (*ExecutionEvents::onVarWrite)(
				const std::string&	value, 
				const char*			type,	// The FLIP data type.
				unsigned			page, 
				unsigned			row
			);

void (*ExecutionEvents::onVarDeclaration)(
	const std::string&	name, 
	const std::string&	value, 
	const char*			type,		// The FLIP data type.
	const char*			storage,	// The FLIP storage class.
	unsigned			page, 
	unsigned			row
);

void (*ExecutionEvents::onVarDestruction)(unsigned page, unsigned row);

//*************************

void (*ExecutionEvents::onArrayDeclaration)(
	unsigned startPage, 
	unsigned startRow, 
	unsigned length
);

void (*ExecutionEvents::onArrayAllocation)(
	unsigned length
);

void (*ExecutionEvents::onArrayDestruction)(
	unsigned startPage, 
	unsigned startRow, 
	unsigned length
);

void (*ExecutionEvents::onArrayIndexing)(
	unsigned	startPage,
	unsigned	startRow,
	unsigned	length,
	unsigned	index
);

//*************************

void (*ExecutionEvents::onRecordDeclaration)(void);
void (*ExecutionEvents::onRecordDestruction)(void);

//*************************

void (*ExecutionEvents::onFunctionEnter)(const char* funname, unsigned line);
void (*ExecutionEvents::onFunctionExit)(const char* funname, unsigned line);
void (*ExecutionEvents::onFunctionUsage)(const char *type, const char *name, unsigned line);

void (*ExecutionEvents::onResult)(const char* value, const char* funname, unsigned line);
void (*ExecutionEvents::onFinish)(const char* funname, unsigned line);

//*************************

void (*ExecutionEvents::onPageOf)(unsigned page, unsigned row, unsigned line);
void (*ExecutionEvents::onRowOf)(unsigned page, unsigned row, unsigned line);
void (*ExecutionEvents::onContentOf)(unsigned page, unsigned row, unsigned line);
void (*ExecutionEvents::onPositionOf)(unsigned page, unsigned row, unsigned line);

void (*ExecutionEvents::onGrantPositions)(
	unsigned page, 
	unsigned row, 
	unsigned size, 
	unsigned line
);

void (*ExecutionEvents::onReleasePositions)(
	unsigned page, 
	unsigned row, 
	unsigned line
);

//*************************

void (*ExecutionEvents::onRegionUsage)(
	unsigned page, 
	unsigned startRow, 
	unsigned endRow, 
	unsigned line
);

void (*ExecutionEvents::onRegionIndexing)(
	unsigned page, 
	unsigned startRow, 
	unsigned endRow, 
	unsigned index
);

//*************************

void		(*ExecutionEvents::onProgramStart)(void);
void		(*ExecutionEvents::onProgramEnd)(void);
void		(*ExecutionEvents::onProgramError)(const char* s);
void		(*ExecutionEvents::onProgramStop)(unsigned line);

int			(*ExecutionEvents::outputFun)(const char *format, ...);
const char*	(*ExecutionEvents::inputFun)(const char* buffer, unsigned len, FILE* fp);
void		(*ExecutionEvents::cleanUpFun)(void);

#endif	// HAS_VISUALIZER