// NotebookMirror.cpp
// Notebook content READ access for the GTK UI.
// GTK visualiser for the FLIP language.
// A. Savidis, October, 2005.
//
#if	defined(HAS_VISUALIZER) && defined(GRAPHICAL_VISUALIZER)

#include <assert.h>
#include "GraphicalUILinkage.h"
#include "NotebookMirror.h"

extern unsigned	MemMakeIndex (unsigned page, unsigned row);

////////////////////////////////////////////////////////////////

NotebookMirror* NotebookMirror::singletonPtr;

void NotebookMirror::SingletonCreate (void) {
	assert(!singletonPtr);
	singletonPtr = new NotebookMirror;
}

void NotebookMirror::SingletonDestroy (void) {
	assert(singletonPtr);
	delete singletonPtr;
	singletonPtr = (NotebookMirror*) 0;
}

NotebookMirror*	NotebookMirror::GetSingleton (void) {
	assert(singletonPtr);
	return singletonPtr;
}

//************************

NotebookMirror::NotebookMirror (void) {
	cellsMutex  = GRAPHICALUI::CreateMutex();
	assert(cellsMutex);
}

NotebookMirror::~NotebookMirror() {
	GRAPHICALUI::DestroyMutex(cellsMutex);
	cellsMutex = 0;
	cells.clear();
}

////////////////////////////////////////////////////////////////

void NotebookMirror::EraseCell (unsigned page, unsigned row) {
	unsigned index = MemMakeIndex(page, row);
	cells.erase(index);
}

NotebookMirror::Cell* NotebookMirror::GetCell (unsigned page, unsigned row) {
	unsigned index = MemMakeIndex(page, row);
	CellMap::iterator i = cells.find(index);
	return i == cells.end() ? (Cell*) 0 : &i->second;
}

NotebookMirror::Cell* NotebookMirror::NewCell (unsigned page, unsigned row) {
	assert(!GetCell(page, row));
	cells[MemMakeIndex(page, row)] = Cell();
	return GetCell(page, row);
}

void NotebookMirror::GrantCellAccess (void) {
	GRAPHICALUI::GrantMutex(cellsMutex);
}

//************************

bool NotebookMirror::IsCellOccupied (unsigned page, unsigned row) {
	return !!GetCell(page, row);
}

//************************

void NotebookMirror::GetCellContent (
			unsigned		page,
			unsigned		row,
			std::string&	name,
			std::string&	value,
			std::string&	type,
			std::string&	storage
		) {
	Cell* cell = GetCell(page, row);
	assert(cell);
	name	= cell->name;
	value	= cell->value;
	type	= cell->type;
	storage	= cell->storage;
}

//************************

void  NotebookMirror::SetCellContent (
		unsigned			page,
		unsigned			row,
		const std::string&	name,
		const std::string&	value,
		const std::string&	type,
		const std::string&	storage
		) {

	Cell cell(name, value, type, storage);
	Cell* p = GetCell(page, row);
	if (!p)
		p = NewCell(page, row);
	assert(p);
	*p = cell;
}

//************************

void NotebookMirror::ReleaseCellAccess (void) {
	GRAPHICALUI::ReleaseMutex(cellsMutex);
}

////////////////////////////////////////////////////////////////

#endif	// HAS_VISUALIZER && GRAPHICAL_VISUALIZER
